from .yams.yams import Yams
from .config.config import Configurator, EnvironmentType
from .modulator.modulator import Modulator
from pathlib import Path

from .server import create_server
dev_mode: bool = False

def main(config: str = None, path: str = None):
    print("\nApplication started:\n")

    if dev_mode:
        print("Starting configuration...\n")
    config = Configurator(dev_mode=dev_mode)
    
    if dev_mode:
        print("\nConfiguration Complete:\n")

    if dev_mode:
        print("\nParsing Yaml File...\n")
    yaml_file = Yams(config.Config)
    yaml_file.parse_file()
    if dev_mode:
        print("\nParse Complete\n")

    if dev_mode:
        print("\nConfiguring WebService...\n")

    mod_importer = Modulator()
    #! Test to see how import modules works
    for layer_key, layer in yaml_file.Layers.items():
        layer_dir = Path(f"{config.Config.DirectoryPath}/{layer}")

        print(f"Layer: {layer_dir}")
        mod_importer.build_modules_from_dir(layer_dir)

    for key, lam in yaml_file.Lambdas.items():
        parts = lam.Handler.split(".")

        lambda_full_path = Path(
            f"{config.Config.DirectoryPath}/{lam.CodeUri}/{parts[0]}.py"
        )
        
        spec, func = mod_importer.build_modules_from_file(lambda_full_path, lam.FunctionName)

        lam.func = func
        lam.spec = spec
        lam.FullPath = lambda_full_path

    create_server(config.Config, yaml_file.Lambdas, yaml_file.Layers)


if __name__ == "__main__":
    main()



from .models import DirObj
from pathlib import Path
from importlib.util import spec_from_file_location, module_from_spec
import sys


class Modulator:
    """
    Builds and imports modules into current application
    """

    def build_modules_from_dir(self, dir_path: Path):
        dir = DirObj(dir_path=dir_path, dir_children=[], modules={})
        self.__build_nodes(dir)
        self.__parse_nodes(dir)

    def build_modules_from_file(self, file_path: Path, func_name: str):
        return self.__import_module(file_path)

    def __build_nodes(self, parent_dir: DirObj):
        print("Building Nodes...")
        dem_strs = parent_dir.dir_path.glob("*")

        for item_path in dem_strs:
            if "__" in item_path.name:
                continue
            print(f"layer_item: {item_path}")

            if item_path.suffix == ".py" and item_path.name != "__init__.py":
                print(
                    f"Added module: {item_path.name} to node: {parent_dir.dir_path.name}"
                )
                parent_dir.modules[item_path.stem] = item_path

            elif item_path.is_dir():
                print(f"is Directory: {item_path}")
                child_dir = DirObj(dir_path=item_path)
                parent_dir.add_child(child_dir)
                self.__build_nodes(child_dir)

    def __parse_nodes(self, node: DirObj):
        print("Parsing Nodes...")

        if node.dir_children is not None and len(node.dir_children) > 0:
            print(f"{node.dir_path.name}: has children")
            for child in node.dir_children:
                if child.modules is not None and len(child.modules) > 0:
                    print(f"child dir: {child.dir_path.name}")
                    self.__parse_nodes(child)

        if node.modules is not None and len(node.modules) > 0:
            print(f"{node.dir_path.name}: has modules")
            for module in node.modules:
                self.__import_module(node.modules[module])

    def __import_module(self, path_str: str, func_name: str | None = None):
        module_path = Path(path_str)

        spec = spec_from_file_location(module_path.stem, module_path)
        func = module_from_spec(spec)
        
        if func_name is not None: 
            sys.modules[func_name] = func  
        else:
            sys.modules[module_path.stem] = func
        
        spec.loader.exec_module(func)

        return (spec, func)



from pydantic import BaseModel
from pathlib import Path

class Config(BaseModel):
    """
    Configuration model class
    """
    
    TemplateName: str
    DirectoryPath: str
    Port: str | int
    MonitorLayers: bool
    Debug: bool
    Host: str


from pathlib import Path
from pydantic import BaseModel, Field
from pydantic.dataclasses import dataclass


class DirObj(BaseModel):
    """
    Model for the root directory object
    """

    dir_path: Path 
    dir_children: list = Field(default_factory=list)
    modules: dict[str, object] = Field(default_factory=dict)
    
    def add_child(self, child_dir: object):
        self.dir_children.append(child_dir)
    


# from .yams.yams import Yams
# from .config.config import Configurator, EnvironmentType
# from .modulator.modulator import Modulator
# from pathlib import Path

# from .server import create_server
# dev_mode: bool = False

# def main(config: str = None, path: str = None):
#     print("\nApplication started:\n")

#     if dev_mode:
#         print("Starting configuration...\n")
#     config = Configurator(dev_mode=dev_mode)
    
#     if dev_mode:
#         print("\nConfiguration Complete:\n")

#     if dev_mode:
#         print("\nParsing Yaml File...\n")
#     yaml_file = Yams(config.Config)
#     yaml_file.parse_file()
#     if dev_mode:
#         print("\nParse Complete\n")

#     if dev_mode:
#         print("\nConfiguring WebService...\n")

#     mod_importer = Modulator()
#     #! Test to see how import modules works
#     for layer_key, layer in yaml_file.Layers.items():
#         layer_dir = Path(f"{config.Config.DirectoryPath}/{layer}")

#         print(f"Layer: {layer_dir}")
#         mod_importer.build_modules_from_dir(layer_dir)

#     for key, lam in yaml_file.Lambdas.items():
#         parts = lam.Handler.split(".")

#         lambda_full_path = Path(
#             f"{config.Config.DirectoryPath}/{lam.CodeUri}/{parts[0]}.py"
#         )
        
#         spec, func = mod_importer.build_modules_from_file(lambda_full_path, lam.FunctionName)

#         lam.func = func
#         lam.spec = spec
#         lam.FullPath = lambda_full_path

#     create_server(config.Config, yaml_file.Lambdas, yaml_file.Layers)


# if __name__ == "__main__":
#     main()



# from setuptools import setup
from distutils.core import setup

setup(
    packages=[
        'SambdaTester',
        'SambdaTester.modulator',
        'SambdaTester.yams',
        'SambdaTester.config'
        ],
    entry_points = {
        'console_scripts': [
            'SambdaTester=SambdaTester:main',
            'bin/SambdaTester=SambdaTester.command:main'
        ]
    },
    version="0.0.13",
    author_email = "tarnold0788@gmail.com"
)

# setup()


from pathlib import Path
import os
import yaml

from ..config.config import EnvironmentType
from ..config.config_models import Config
from .models.yaml_model import Lambda

class Yams(object):
    """
    Class that parses and represents the AWS Sam template file
    """
    ConfigObj: Config
    Lambdas: dict = {}
    Layers: dict = {}
    
    def __init__(self, config: Config) -> None:
        """
        Constructor for Yams class

        Args:
            work_dir_path (str): path to input file, if none is provided this will default to the users working directory.
            config (Config): Configuration object of application
        """
        
        # Sets the configuration object that is referenced when parsing the template
        self.ConfigObj = config
    
    def parse_file(self):
        if os.environ.get("ENV") != EnvironmentType.Prod:
            print(f"FileName: {self.ConfigObj}")
            
        full_path = Path(self.ConfigObj.DirectoryPath).joinpath(self.ConfigObj.TemplateName)
            
        with open(full_path, "r") as file:
            yaml.add_multi_constructor("!", self.__build_loader)
            yaml.add_multi_constructor("Ref:", self.__build_loader)
            
            docs = yaml.load_all(file, Loader=yaml.FullLoader)
        
            for doc in docs:
                global_section: dict
                globe_func: dict
                resources: dict = doc["Resources"]
                handler: str = ""
                
                if "Globals" in doc:
                    global_section = doc["Globals"]
                    
                    if "Function" in global_section:
                        globe_func = global_section["Function"]
                        
                        if "Handler" in globe_func:
                            handler = globe_func["Handler"]
                
                for key, aws_lambda in resources.items():
                    if aws_lambda["Type"] == "AWS::Serverless::LayerVersion":
                        props = aws_lambda["Properties"]
                        self.Layers[key] = props["ContentUri"]
                    
                    if aws_lambda["Type"] == "AWS::Serverless::Function":
                        props = aws_lambda["Properties"]
                        api_props = props["Events"]["RestApi"]["Properties"]
                        elements = {k:api_props[k] for k in api_props if k !="RestApiId" and k !="Auth"}
                        
                        name: str = props["FunctionName"]
                        elements["FunctionName"] = name
                        code_uri: str = props["CodeUri"]
                        elements["CodeUri"] = code_uri
                        
                        if handler == "" and "Handler" in props:
                            handler = props["Handler"]
                        
                        assert handler != "", f"Error retrieving handler from lambda {key}"
                        
                        elements["Handler"] = handler
                        
                        self.Lambdas[name] = Lambda(**elements)    
            
    def __build_loader(self, loader, suffix, node):
        if isinstance(node, yaml.ScalarNode):
            constructor = loader.__class__.construct_scalar
        elif isinstance(node, yaml.SequenceNode):
            constructor = loader.__class__.construct_sequence
        elif isinstance(node, yaml.MappingNode):
            constructor = loader.__class__.construct_mapping

        data = constructor(loader, node)
        
        return data
        
        
        


from .app import main
from .yams import Yams
from .config import Config


# config_path: str = ""

# try:
#     opt, bad_arg = getopt.getopt(sys.argv[1], "c:m:")
#     bad_opt, arg = getopt.getopt(sys.argv[0], "c:m:")
    
#     print(len(opts))
#     for arg in args:
#         print(f"args: {args}")
    
#     for opt, arg in opts:
        
#         print(f"{opt}")
#         if opt == "-c":
#             config_path = arg
    
#     assert config_path != "", "Must include option -c to specify config file path"    
#     main(config_path)
            
# except ValueError as ve:
#     print(ve)

def execute(config_path:str):
    main(config_path)